//
// GameX - Includes used by GameX
// 
// GameX Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source License. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 
// this is a version of GameX that has been modified (by Justin Pease)


#ifndef GAMEX_GLOBAL_DEF_H_INCLUDED
#define GAMEX_GLOBAL_DEF_H_INCLUDED

//#define CLEANMEM_DISABLE // only uncomment this line if you REALLY don't want automatic memory leak/corruption checking


// necessary libraries
#pragma comment(lib,"kernel32.lib") // auto-link with required windows libraries
#pragma comment(lib,"user32.lib") // (just in case the linker settings are missing these)
#pragma comment(lib,"gdi32.lib")
#pragma comment(lib,"ddraw.lib") // auto-link DDRAW.LIB
#pragma comment(lib,"dinput.lib") // auto-link DINPUT.LIB
#pragma comment(lib,"dsound.lib") // auto-link DSOUND.LIB
#pragma comment(lib,"dxguid.lib") // auto-link DXGUID.LIB
#pragma comment(lib,"strmiids.lib") // auto-link STRMIIDS.LIB

#if !defined(_LIB) && !defined(NOGAMEXLIB) // if not building the GameX library
	#pragma comment(lib,"gamex.lib") // auto-link with GameX
#endif

// some forward declarations to avoid further includes for everything
typedef struct IDirectDrawSurface7       *LPDIRECTDRAWSURFACE7;
typedef struct IDirectDraw7              *LPDIRECTDRAW7;
struct IDirectMusicPerformance;
struct IDirectMusicLoader;
typedef struct _D3DDeviceDesc7 *LPD3DDEVICEDESC7;
typedef struct IDirect3D7 *LPDIRECT3D7;
typedef struct IDirect3DDevice7       *LPDIRECT3DDEVICE7;
typedef struct IDirect3DVertexBuffer7 *LPDIRECT3DVERTEXBUFFER7;
typedef struct _DDPIXELFORMAT DDPIXELFORMAT;
typedef DDPIXELFORMAT * LPDDPIXELFORMAT;
typedef struct IDirectSound                 *LPDIRECTSOUND;
typedef struct IDirectSoundBuffer           *LPDIRECTSOUNDBUFFER;
struct IDirectMusicSegment;
struct IDirectMusicSegmentState;
struct IGraphBuilder;
struct IMediaControl;
struct IMediaPosition;
struct IBasicAudio;
struct IMediaEventEx;
struct IBasicVideo;
struct IVideoWindow;
struct IMediaEventEx;
struct IDirectDrawVideo;
typedef long HRESULT;
struct HINSTANCE__; typedef struct HINSTANCE__ *HINSTANCE;
struct HWND__; typedef struct HWND__ *HWND;
typedef struct tagRECT RECT;
#ifdef UNICODE
typedef struct IDirectInputDeviceW *LPDIRECTINPUTDEVICE;
typedef struct IDirectInputW *LPDIRECTINPUT;
#else
typedef struct IDirectInputDeviceA *LPDIRECTINPUTDEVICE;
typedef struct IDirectInputA *LPDIRECTINPUT;
#endif
typedef struct _DIMOUSESTATE *LPDIMOUSESTATE;
typedef struct _DDSURFACEDESC2 *LPDDSURFACEDESC2;

struct ColorX;
class CameraX;
class ImageX;
struct GameXDrawState;
typedef long DrawFlags;
typedef long GameXInitFlags;
typedef int GameXStatus;
class File;
//The below typedef overwrites standard C type, why?
//struct _iobuf; typedef struct _iobuf FILE;
class Buffer;
class ImageExt; // forward referencing
typedef int KeyID;
typedef int MouseButtonID;
class MusicX;
class MovieX;
class SoundX;
typedef long SceneSortMode;
typedef int SoundPlayMode;
template <typename type> struct Vec2;
template <typename type> struct Vec3;
template <typename type> struct Vec4;
typedef Vec3<float> Vec3f;
typedef struct _FVFTLVERTEX FVFTLVERTEX;
//class TextPrinter;
//class Debugging;
//class GameXClass;
extern class Debugging debug;
extern class GameXClass GameX;
extern class TextPrinter tp;

class cERROR {public: int _;}; extern cERROR GAMEX_MAIN_MUST_BE_DEFINED_IN_YOUR_MAIN_CPP_FILE_WHICH_MUST_DIRECTLY_INCLUDE_GAMEX_H;

// hack needed for safe rand support
extern long z_gamerand;
extern long z_drawrand;
extern bool z_isdrawing;
extern bool z_wasdrawing;
#include <cstdlib>
#define rand() GameX.GetRand()
#define srand(x) GameX.SetRandomSeed(x)
#define z_pushDrawing() z_wasdrawing = z_isdrawing, z_isdrawing = true
#define z_popDrawing() z_isdrawing = z_wasdrawing
#define z_GameDraw() z_LoadQueuedImages(), z_pushDrawing(), GameDraw(), z_DrawDefaultCursor(), z_popDrawing()

// defaults used internally:
#define DEFAULT 	-99999 // reserved value that signifies either a default color or image rectangle full expansion (or rarely, non-scaled centered changed to DEFAULT2)
#define DEFAULT2	-99998.0f // reserved value that signifies image rectangle non-scaled expansion
#define FULLRECT	(*((RECT *)0)) // dummy value to mean a rectangle that fills up the entire available space (0,0,DEFAULT,DEFAULT)

// windows versions (allowed to be overriden)
#ifndef _WIN32_WINDOWS
#	define _WIN32_WINDOWS 0x0410 // windows 98
#endif
#ifndef _WIN32_WINNT
#	define _WIN32_WINNT 0x0410 // windows 98
#endif
#ifndef WINVER
#	define WINVER 0x0500 // windows NT
#endif

// globally used GameX includes
#include "gamex-cleanmem.h" // memory management
#include "gamex-debug.h" // debug utility commands, so you can do things like  debug.Output("something");
#include "gamex-defines.h" // defines important GameX constants and macros
#include "gamex-vector.h" // vector support
#include "gamex-utilities.h" // general classes for GameX functions

// frequently useful includes
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

// useful std stuff
#include <vector>
#include <list>
#include <stack>
#include <queue>
using namespace std;



#define WM_DIRECTSHOW_EVENT (WM_APP+1) // custom WindowFunc callback function for music events


#endif
