//
// GameX 5 - Primary Header File
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source License. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 


#ifndef GAMEX_H_INCLUDED
#define GAMEX_H_INCLUDED

#include "gamex-global-def.h" // globally necessary includes and defines
#include "gamex-image.h" // support for loading and converting graphics
#include "gamex-sound.h" // support for loading sounds and music
#include "gamex-matrix.h" // matrix support
#include "gamex-camera.h" // 3D camera support
#include "gamex-text.h" // graphical text support
#include "gamex-win-dx.h" // GameX itself

// Only once, when being included from the main file which defines GAMEX_MAIN,
// (this is the only way to allow this to happen only once per entire build --
//  include guards and #pragma once only prevent happening more than once per .cpp file being compiled, which isn't good enough)
#ifdef GAMEX_MAIN

	// if GAMEX_MAIN is defined in more than one .cpp or .c file, there will be a "redefinition" error anyway,
	// so we might as well make the error message say what the real error is with this
	typedef int bERROR; bERROR GAMEX_MAIN_MUST_ONLY_BE_DEFINED_IN_ONE_CPP_FILE;

	// if GAMEX_MAIN isn't defined in any .cpp file, there will be an "unresolved external symbol" linker error anyway,
	// so we might as well make the error message say what the real error is with this
	cERROR GAMEX_MAIN_MUST_BE_DEFINED_IN_YOUR_MAIN_CPP_FILE_WHICH_MUST_DIRECTLY_INCLUDE_GAMEX_H;

	// create the exported global objects:
	// (done here to trick IntelliSense into being more likely to work...)
	Debugging debug;  // global Debugging object
	GameXClass GameX; // global GameX object
	TextPrinter tp;   // global TextPrintexr object (seperate from GameX only as a backwards compatibility hack)

#endif

#endif