//
// GameX
// Ship Demo - Main Program
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source License. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

#define GAMEX_MAIN							// The main .cpp file must define GAMEX_MAIN before including GameX
#include "gamex.h"							// Include GameX


struct Circle2f
{
	Vec2f center;
	float radius;
};


// the default case
template<class Object1, class Object2>
inline bool CheckCollision(const Object1& obj1, const Object2& obj2, std::vector<Vec2f>* hitLocations=NULL)
{
	return false; // all unimplemented pairs will do this
}

// specialization for each supported pair of collidable types:

// check whether 2 circles intersect
// note: you'd have to make a Circle class, because it isn't already defined in GameX
template<> bool CheckCollision(const Circle2f& obj1, const Circle2f& obj2, std::vector<Vec2f>* hitLocations)
{
	if(obj1.center.DistSq(obj2.center) < Square(obj1.radius + obj2.radius))
	{
		if(hitLocations)
			hitLocations->push_back((obj1.center + obj2.center) * 0.5f);
		return true;
	}
	return false;
}

// check whether a point is inside a circle
template<> bool CheckCollision(const Circle2f& obj1, const Vec2f& obj2, std::vector<Vec2f>* hitLocations)
{
	if(obj1.center.DistSq(obj2) < Square(obj1.radius))
	{
		if(hitLocations)
			hitLocations->push_back(obj2);
		return true;
	}
	return false;
}
template<> inline bool CheckCollision(const Vec2f& obj1, const Circle2f& obj2, std::vector<Vec2f>* hitLocations)
{ return CheckCollision(obj2,obj1,hitLocations); }

// check whether 2 line segments intersect
// note: this one should work already
template<> bool CheckCollision(const LineSeg2f& obj1, const LineSeg2f& obj2, std::vector<Vec2f>* hitLocations)
{
	static Vec2f collisionPoint;
	const bool collided = obj1.testIntersection(obj2, collisionPoint);
	if(collided && hitLocations)
		hitLocations->push_back(collisionPoint);
	return collided;
}

// etc.







#include "shipdemox-photons.hpp"			// Include for Photons Class
#include "shipdemox-ship.hpp"				// Include for Ship Class

											//*** Global Variables
Ship blueShip, redShip;						// Red and Blue ship objects
ImageX backImg;								// Background Image

// Required GameX initialize function (called when program starts)
// this function should initialize GameX and prepare anything else we need
void GameInit (void)
{											// Initialize GameX itself (required)

	Circle2f a;
	LineSeg2f b;
	bool aa = CheckCollision(a,a);
	bool ab = CheckCollision(a,/*b*/Vec2f(0,0));
	bool bb = CheckCollision(b,b);

	GameX.Initialize("ShipDemoX", VIDEO_ALLOWREFRESHSYNC, 800, 400);

	g_photons.Initialize();					// Initialize Photons
	redShip.Initialize(1);					// Initialize Red Ship  (controlled by WASDX keys)
	blueShip.Initialize(0);					// Initialize Blue Ship (controlled by arrow keys + space)
	backImg.Load("Images\\ice.jpg");		// Load the background
}

// Required GameX uninitialize function (called when program exits)
// this function should free any dynamic memory the game has allocated
void GameCleanup (void)
{
	int p = 5;
}

// Required GameX main function (called repeatedly while game is running)
// this function should update the game state but should NOT draw that state
void GameRun (void)
{											// Update Ships & Photons
	blueShip.Update();						// (call respective class member functions)
	redShip.Update();
	g_photons.Update();	

	blueShip.CollideCheck(&redShip);		// Ship-to-Ship Collision Detection
}

// Required GameX draw function (called repeatedly while game is running)
// this function should draw the game state and do as little else as possible
void GameDraw (void)
{										// Draw Background (ice), offset opposite of ships
	GameX.SetDrawShading(ColorX(255,255,255,255)); // red,green,blue,alpha, each out of 255
	GameX.DrawTiled(&backImg, redShip.GetPos()*redShip.GetMass() + blueShip.GetPos()*blueShip.GetMass());
	blueShip.Draw();					// Draw Red and Blue ships
	redShip.Draw();
	g_photons.Draw();						// Draw Photons
}
